using System;
using System.Collections.Generic;
using System.Text;
using IADSDATAFILEINTERFACELib;
using IADSTIMELib;

namespace DataTesterCS
{
    class Program
    {
        static void Main(string[] args)
        {
            IadsTimeClass Time = new IadsTimeClass();

            DataFileClass IADSFile = new DataFileClass();
            
            //
            // Read Tests
            //
            object irigTime10 = new Object[1000] ;
            IADSFile.CreateIadsTimeArray(100, ref irigTime10);

            IADSFile.OpenExisting(@"\PostFlightData\IADSTestData\NewIadsData\FixedDataSize\PeriodicDiscrete", "Parm1");
            //IADSFile.OpenExisting("d:\\", "Parm1");
            IadsTime StartTime = (IadsTime)IADSFile.StartTimeVB;
            Int64 sTime = StartTime.TotalNanoseconds;

            // 1) Read a periodic file (F32) file return Irig Time array values and do a sequential read (-1 for time)
            UInt32[] uiData = new UInt32[1000];
            //object outData = new object[1000];
            object irigTime = new object[1000]; // this will fill the time array
            object outData = (object)uiData;
            int actualPoints = 0;
            object anp = null; // ActualPoints now needs to be passed as an object

            IADSFile.ReadData(IADSFile.StartTimeVB, 1000, ref outData, ref anp, ref irigTime);
            actualPoints = (int)anp; // and casted after to an integer.

            uiData = (UInt32[])outData;

            // Try the iadsTime to int conversion routines.
            UInt32[] timeHigh = new UInt32[1000];
            UInt32[] timeLow = new UInt32[1000];
            for (int n = 0; n < 1000; n++)
            {
              timeHigh[n] = 0; timeLow[n] = 0;
            }
            object timeInHigh = (object)timeHigh;
            object timeInLow = (object)timeLow;
            IADSFile.FromIadsTimeToInt( ref irigTime, ref timeInHigh, ref timeInLow);

            timeHigh = (UInt32[])timeInHigh;
            timeLow = (UInt32[])timeInLow;

            for (int n = 0; n < 1000; n++)
            {
              Int64 nanoTime = timeHigh[n];
              nanoTime = (nanoTime <<= 32) | timeLow[n];
            }
           
            // Do a sequential read by setting start time to -1
            StartTime.TotalNanoseconds = -1;
            IADSFile.ReadData((IADSDATAFILEINTERFACELib.IIadsTime)StartTime, 1000, ref outData, ref anp, ref irigTime);
            actualPoints = (int)anp; // and casted after to an integer.
            IADSFile.Close();

            // 2) Read a periodic file (F32) file no Irig Time array values)
            IADSFile.OpenExisting(@"\PostFlightData\IADSTestData\NewIadsData\FixedDataSize\PeriodicDiscrete", "Parm1");
            object irigTime2 = new object[0]; // This will not fill the time array
            actualPoints = 0;
            IADSFile.ReadData(IADSFile.StartTimeVB, 1000, ref outData, ref anp, ref irigTime2);
            actualPoints = (int)anp; // and casted after to an integer.
            IADSFile.Close();

            // 3) Read an aperiodic file (F32) file no Irig Time array values)
            IADSFile.OpenExisting(@"D:\PostFlightData\IADSTestData\NewIadsData\FixedDataSize\AsyncFloat", "Parm1");
            IADSFile.ReadData(IADSFile.StartTimeVB, 1000, ref outData, ref anp, ref irigTime);
            actualPoints = (int)anp; // and casted after to an integer.
            IADSFile.Close();
            
            //
            // Write Tests
            //

            RunUserCode(IADSFile);

            // 1) Write a periodic file - the irigTimeArray doesn't matter time values are internally calculated
            IADSFile.Create(@"c:\", "Parm10");
            IADSFile.SampleRate = 100.0;
            IADSFile.DataFormat = IadsDataFormat.IadsFloat32Type;
            IADSFile.DataType = IadsDataType.IadsPeriodicType;
            StartTime.TotalNanoseconds = sTime;
            IADSFile.StartTimeVB = (IADSDATAFILEINTERFACELib.IIadsTime)StartTime; // use the start time from the opened file
            IADSFile.WriteHeader();
            object irigTime3 = new object[0]; // this will not fille the time array and just return data
            float[] data = new float[1000];
            for (int n = 0; n < 1000; n++) data[n] = (float)n;
            object dataIn = (object)data;
            actualPoints = 0;
            IADSFile.AppendData(1000, ref dataIn, ref anp, ref irigTime3);
            IADSFile.AppendData(1000, ref dataIn, ref anp, ref irigTime3);
            IADSFile.Close();

            // 2) Write an Aperiodic file - the irigTimeArray time values come from the user, this will create an index time file
            IADSFile.Create(@"c:\", "Parm20");
            IADSFile.SampleRate = 0.0;
            IADSFile.DataFormat = IadsDataFormat.IadsFloat32Type;
            IADSFile.DataType = IadsDataType.IadsAsyncType;
            StartTime.TotalNanoseconds = sTime;
            IADSFile.StartTimeVB = (IADSDATAFILEINTERFACELib.IIadsTime)StartTime; // use the start time from the opened file
            IADSFile.WriteHeader();

            // Generate some date
            for (int n = 0; n < 1000; n++) data[n] = (float)n+10;
            dataIn = (object)data;

            // Generate some sim times for the AppendData routine
            object iadsTime = new object[1000];
            Int64 time = 86400000000000;
            // Need to crack the 64 bit time into 2 integer 32 bit arrays of the low and high parts
            Int32[] highData = new Int32[1000] ;
            Int32[] lowData = new Int32[1000] ;
            for( int n=0; n < 1000; n++ )
            {
              lowData[n] = (Int32)(time&0xffffffff) ;
              highData[n] = (Int32)(time>>32) ;
              time = time + (n*50000); // These would be the times in your program
            }
            // reference objects
            object highIn = (object)highData ;
            object lowIn = (object)lowData ;
            // Call this routine to make an array of IADS Time objects
            IADSFile.FromIntToIadsTime( ref highIn, ref lowIn, ref iadsTime ) ;

            actualPoints = 0;
            // Need to make sure you populate the times
            IADSFile.AppendData(1000, ref dataIn, ref anp, ref iadsTime);

            IADSFile.Close();
        }
        static int RunUserCode(DataFileClass IADSFile)
        {
            IadsTime iadsStartTime = new IadsTimeClass();
            iadsStartTime.TotalNanoseconds = 86400000000000; // GetLongTime(DateTime.Now);
            string startTimeString = iadsStartTime.TimeString;
            //DataFileClass IADSFile = new DataFileClass();
            Int64[] InTimes = new Int64[1000];
            Int32[] myData = new Int32[1000];
            Int32[] lowData = new Int32[1000];
            Int32[] highData = new Int32[1000];
            object objHighData = new Int32[1000];
            object objLowData = new Int32[1000];
            object objInData = new object[1000];
            object irigTime = new object[1000];
            IADSFile.SampleRate = 0.0;
            IADSFile.DataType = IadsDataType.IadsAsyncType;
            IADSFile.Create("C:\\", "PARAM50");
            IADSFile.DataFormat = IadsDataFormat.IadsInt32Type;
            IADSFile.StartTimeVB = (IADSDATAFILEINTERFACELib.IIadsTime)iadsStartTime;
            IADSFile.WriteHeader();
            int actualPoints = 0;
            for (int x = 0; x < 1000; x++)
            {
                InTimes[x] = 86400000000000 + (x*50000); // GetLongTime(DateTime.Now);
                highData[x] = (Int32)(InTimes[x] >> 32);
                lowData[x] = (Int32)(InTimes[x] & 0xffffffff);
                myData[x] = x;
                //Thread.Sleep(10);
            }

            objHighData = (object)highData;
            objLowData = (object)lowData;
            IADSFile.FromIntToIadsTime(ref objHighData, ref objLowData, ref irigTime);
            objInData = (object)myData;
            object anp=null;
            IADSFile.AppendData(1000, ref objInData, ref anp, ref irigTime);
            actualPoints = (int)anp;
            IADSFile.Close();

            return 1;
        }

        void ShowHeader(DataFileClass IadsFile)
        {
            IadsFile.ReadHeader();
            IadsDataType type = IadsFile.DataType;
            int dataSize = IadsFile.DataSize;
            IadsTime StartTime, EndTime;
            StartTime = (IadsTime)IadsFile.StartTimeVB;
            Int64 totalNano = StartTime.TotalNanoseconds;
            EndTime = (IadsTime)IadsFile.EndTimeVB;
            Int64 totalNano2 = EndTime.TotalNanoseconds;
        }
    }
}
